-- Last modification date: 2018-11-13 16:03:10.45

CREATE EXTENSION IF NOT EXISTS "uuid-ossp";
DROP SCHEMA IF EXISTS med_img CASCADE;
CREATE SCHEMA med_img;

-- tables
-- Table: Hospital
CREATE TABLE med_img.Hospital (
    id serial  NOT NULL,
    type varchar(20)  NOT NULL,
    director varchar(70)  NOT NULL,
    tel_central varchar(15)  NOT NULL,
    tels varchar[]  NOT NULL,
    location_id int  NOT NULL,
    CONSTRAINT Hospital_pk PRIMARY KEY (id)
);

COMMENT ON TABLE med_img.Hospital IS 'Table that stores information about the hospitals';
COMMENT ON COLUMN med_img.Hospital.id IS 'The hospital identifier and primary key';
COMMENT ON COLUMN med_img.Hospital.type IS 'The type of hospital. In Costa Rica, there are 6 types of hospitals';
COMMENT ON COLUMN med_img.Hospital.director IS 'The name of the hospital''''s director';
COMMENT ON COLUMN med_img.Hospital.tel_central IS 'Telephonic central of the hospital';
COMMENT ON COLUMN med_img.Hospital.tels IS 'Telephone numbers of the hospital';
COMMENT ON COLUMN med_img.Hospital.location_id IS 'The identifier of the hospital location tuple';

-- Table: Instance
CREATE TABLE med_img.Instance (
    uuid uuid  NOT NULL DEFAULT uuid_generate_v4(),
    series_uuid uuid  NOT NULL,
    comment text  NULL,
    original boolean  NOT NULL DEFAULT true,
    image pgcv_core.ndarray_int4  NOT NULL,
    created_at timestamp  NOT NULL DEFAULT current_timestamp,
    updated_at timestamp  NOT NULL DEFAULT current_timestamp,
    active boolean  NOT NULL DEFAULT true,
    CONSTRAINT Instance_pk PRIMARY KEY (uuid)
);

COMMENT ON TABLE med_img.Instance IS 'This table stores basic instance/image information inspired by the DICOM standard.';
COMMENT ON COLUMN med_img.Instance.uuid IS 'Universally unique identifier of the instance in the database.';
COMMENT ON COLUMN med_img.Instance.series_uuid IS 'Reference to the uuid of the series.';
COMMENT ON COLUMN med_img.Instance.original IS 'Describes whether an image pixel values were based on source data or have been derived in some manner from the pixel value of one or more other images';
COMMENT ON COLUMN med_img.Instance.image IS 'The datatype containing the image data. Refer to the pgcv PostgreSQL extension.';
COMMENT ON COLUMN med_img.Instance.created_at IS 'Datetime the image pixel data creation started.';
COMMENT ON COLUMN med_img.Instance.updated_at IS 'Datetime the image pixel data was last updated';
COMMENT ON COLUMN med_img.Instance.active IS 'Defines whether an instance is active';

-- Table: Location
CREATE TABLE med_img.Location (
    id serial  NOT NULL,
    state varchar(35)  NOT NULL,
    address varchar(100)  NOT NULL,
    lat_lng decimal(9,6)[2]  NOT NULL,
    CONSTRAINT Location_pk PRIMARY KEY (id)
);

COMMENT ON TABLE med_img.Location IS 'Location information of the hospitals';
COMMENT ON COLUMN med_img.Location.id IS 'The identifier of each location tuple';
COMMENT ON COLUMN med_img.Location.state IS 'The state/province where the hospital is located';
COMMENT ON COLUMN med_img.Location.address IS 'The given street address of the hospital';
COMMENT ON COLUMN med_img.Location.lat_lng IS 'A latitude and longitude tuple indicating the hospital''''s coordinates';

-- Table: Offers
CREATE TABLE med_img.Offers (
    hospital_id int  NOT NULL,
    services_id int  NOT NULL,
    CONSTRAINT Offers_pk PRIMARY KEY (hospital_id,services_id)
);

COMMENT ON TABLE med_img.Offers IS 'Relationship that stores which services are offered by which hospitals';
COMMENT ON COLUMN med_img.Offers.hospital_id IS 'Identifier of the hospital that offers the service';
COMMENT ON COLUMN med_img.Offers.services_id IS 'Identifier of said service';

-- Table: Patient
CREATE TABLE med_img.Patient (
    uuid uuid  NOT NULL DEFAULT uuid_generate_v4(),
    given_name varchar(35)  NOT NULL,
    family_name varchar(35)  NOT NULL,
    other_ids jsonb  NULL,
    email varchar(320)  NULL,
    address jsonb  NOT NULL,
    sex smallint  NOT NULL,
    birthdate date  NOT NULL,
    created_at timestamp  NOT NULL DEFAULT current_timestamp,
    updated_at timestamp  NOT NULL DEFAULT current_timestamp,
    active boolean  NOT NULL DEFAULT true,
    CONSTRAINT Patient_pk PRIMARY KEY (uuid)
);

COMMENT ON TABLE med_img.Patient IS 'This table stores basic patient information inspired by the DICOM standard.';
COMMENT ON COLUMN med_img.Patient.uuid IS 'Universally unique identifier of the patient in the database.';
COMMENT ON COLUMN med_img.Patient.given_name IS 'The given name of the person.';
COMMENT ON COLUMN med_img.Patient.family_name IS 'The family name of the person, usually the last name.';
COMMENT ON COLUMN med_img.Patient.other_ids IS 'JSON array of alternate identifiers for the patient.';
COMMENT ON COLUMN med_img.Patient.email IS 'Email address of the patient';
COMMENT ON COLUMN med_img.Patient.address IS 'JSON object containing information about the region of the patient, for instance, country, state/province, city';
COMMENT ON COLUMN med_img.Patient.sex IS 'Sex of the patient, according to the ISO/IEC 5218: Codes for the representation of human sexes.';
COMMENT ON COLUMN med_img.Patient.birthdate IS 'Birthdate of the patient';
COMMENT ON COLUMN med_img.Patient.created_at IS 'Date and time when the patient was created';
COMMENT ON COLUMN med_img.Patient.updated_at IS 'Date and time when the patient was last updated';
COMMENT ON COLUMN med_img.Patient.active IS 'Defines whether the patient is active';

-- Table: Region
CREATE TABLE med_img.Region (
    instance_uuid uuid  NOT NULL,
    method varchar(70)  NOT NULL,
    props pgcv_core.regionprops  NOT NULL,
    category varchar(70)  NULL,
    created_at timestamp  NOT NULL DEFAULT current_timestamp,
    updated_at timestamp  NOT NULL DEFAULT current_timestamp,
    active boolean  NOT NULL DEFAULT true,
    CONSTRAINT Region_pk PRIMARY KEY (instance_uuid,method,props)
);

COMMENT ON TABLE med_img.Region IS 'This table stores information about the regions found in an image using a segmentation method';
COMMENT ON COLUMN med_img.Region.instance_uuid IS 'Reference to the uuid of the instance.';
COMMENT ON COLUMN med_img.Region.method IS 'Method used to extract the region properties. This allows for different segmentation functions';
COMMENT ON COLUMN med_img.Region.props IS 'The datatype containing properties of the region. Refer to the pgcv PostgreSQL extension.';
COMMENT ON COLUMN med_img.Region.category IS 'Class of region according to a classifier';
COMMENT ON COLUMN med_img.Region.created_at IS 'Datetime the region information was extracted from the image';
COMMENT ON COLUMN med_img.Region.updated_at IS 'Datetime the region was last updated';
COMMENT ON COLUMN med_img.Region.active IS 'Defines whether the region is active';

-- Table: Schedule
CREATE TABLE med_img.Schedule (
    hospital_id int  NOT NULL,
    category varchar(35)  NOT NULL,
    name varchar(70)  NOT NULL,
    "when" jsonb  NOT NULL,
    CONSTRAINT Schedule_pk PRIMARY KEY (hospital_id,category,name)
);

COMMENT ON TABLE med_img.Schedule IS 'Attention schedules of each hospital';
COMMENT ON COLUMN med_img.Schedule.hospital_id IS 'Hospital referencing the schedule';
COMMENT ON COLUMN med_img.Schedule.category IS 'Category of the schedule, e.g.: visits';
COMMENT ON COLUMN med_img.Schedule.name IS 'Name of the schedule';
COMMENT ON COLUMN med_img.Schedule."when" IS 'JSON array containing schedule objects. Each schedule objects contains a list of days and a time interval';

-- Table: Series
CREATE TABLE med_img.Series (
    uuid uuid  NOT NULL DEFAULT uuid_generate_v4(),
    study_uuid uuid  NOT NULL,
    laterality char(1)  NULL,
    description varchar(255)  NOT NULL,
    body_part varchar(35)  NOT NULL,
    created_at timestamp  NOT NULL DEFAULT current_timestamp,
    updated_at timestamp  NOT NULL DEFAULT current_timestamp,
    active boolean  NOT NULL DEFAULT true,
    hospital_id int  NOT NULL,
    CONSTRAINT Series_pk PRIMARY KEY (uuid)
);

COMMENT ON TABLE med_img.Series IS 'This table stores basic series information inspired by the DICOM standard.';
COMMENT ON COLUMN med_img.Series.uuid IS 'Universally unique identifier of the series in the database.';
COMMENT ON COLUMN med_img.Series.study_uuid IS 'Reference to the uuid of the study.';
COMMENT ON COLUMN med_img.Series.laterality IS 'Laterality of (paired) body part examined. Required if the body part examined is a paired structure. R = right, L = left';
COMMENT ON COLUMN med_img.Series.description IS 'User provided description of the Series';
COMMENT ON COLUMN med_img.Series.body_part IS 'Text description of the part of the body examined';
COMMENT ON COLUMN med_img.Series.created_at IS 'Date and time when the series started';
COMMENT ON COLUMN med_img.Series.updated_at IS 'Date and time when the series was last updated';
COMMENT ON COLUMN med_img.Series.active IS 'Defines whether the study is active';
COMMENT ON COLUMN med_img.Series.hospital_id IS 'Hospital where the series were captured';

-- Table: Services
CREATE TABLE med_img.Services (
    id serial  NOT NULL,
    service_area varchar(35)  NOT NULL,
    service_name varchar(70)  NOT NULL,
    CONSTRAINT Services_pk PRIMARY KEY (id)
);

COMMENT ON TABLE med_img.Services IS 'Services of the hospitals';
COMMENT ON COLUMN med_img.Services.id IS 'The identifier of the service';
COMMENT ON COLUMN med_img.Services.service_area IS 'The service area that includes particular services';
COMMENT ON COLUMN med_img.Services.service_name IS 'The given service name';

-- Table: Study
CREATE TABLE med_img.Study (
    uuid uuid  NOT NULL DEFAULT uuid_generate_v4(),
    patient_uuid uuid  NOT NULL,
    description varchar(255)  NOT NULL,
    summary jsonb  NULL,
    created_at timestamp  NOT NULL DEFAULT current_timestamp,
    updated_at timestamp  NOT NULL DEFAULT current_timestamp,
    active boolean  NOT NULL DEFAULT true,
    hospital_id int  NOT NULL,
    CONSTRAINT Study_pk PRIMARY KEY (uuid)
);

COMMENT ON TABLE med_img.Study IS 'This table stores basic study information inspired by the DICOM standard.';
COMMENT ON COLUMN med_img.Study.uuid IS 'Universally unique identifier of the study in the database.';
COMMENT ON COLUMN med_img.Study.patient_uuid IS 'Reference to the uuid of the patient.';
COMMENT ON COLUMN med_img.Study.description IS 'Description or classification of the Study performed.';
COMMENT ON COLUMN med_img.Study.summary IS 'Stores information about the overall study and reporting information, for instance, the BI-RADS category';
COMMENT ON COLUMN med_img.Study.created_at IS 'Date and time when the study started';
COMMENT ON COLUMN med_img.Study.updated_at IS 'Date and time when the study was last updated';
COMMENT ON COLUMN med_img.Study.active IS 'Defines whether the study is active';
COMMENT ON COLUMN med_img.Study.hospital_id IS 'Hospital where the study started';

-- Table: Treats
CREATE TABLE med_img.Treats (
    patient_uuid uuid  NOT NULL,
    hospital_id int  NOT NULL,
    visit_datetime timestamp  NOT NULL,
    CONSTRAINT Treats_pk PRIMARY KEY (patient_uuid,hospital_id,visit_datetime)
);

COMMENT ON TABLE med_img.Treats IS 'Table that registers every-time a patient visits an hospital, meaning, everytime a patient is treated in a given hospital';
COMMENT ON COLUMN med_img.Treats.patient_uuid IS 'The patient identifier';
COMMENT ON COLUMN med_img.Treats.hospital_id IS 'The hospital identifier';
COMMENT ON COLUMN med_img.Treats.visit_datetime IS 'A timestamp of when the patient was treated ';

-- views
-- View: instance_thumbnails
CREATE MATERIALIZED VIEW med_img.instance_thumbnails AS
SELECT
       uuid,
       series_uuid,
       pgcv_core.thumbnail_uri_base64(image) AS thumbnail_uri,
       created_at
FROM med_img.Instance
WHERE active = true;

COMMENT ON MATERIALIZED VIEW med_img.instance_thumbnails IS 'Materialized view of the Instance thumbnails. This view provides access to the data URIs of each Instance';
COMMENT ON COLUMN med_img.instance_thumbnails.uuid IS 'Universally unique identifier of the instance in the database.';
COMMENT ON COLUMN med_img.instance_thumbnails.series_uuid IS 'Reference to the uuid of the series.';
COMMENT ON COLUMN med_img.instance_thumbnails.thumbnail_uri IS 'Data URIs of the thumbnail of each Instance';
COMMENT ON COLUMN med_img.instance_thumbnails.created_at IS 'Datetime the Instance was created';

CREATE OR REPLACE FUNCTION med_img.refreshInstanceThumbnails()
  RETURNS TRIGGER AS $_$
BEGIN
  REFRESH MATERIALIZED VIEW med_img.instance_thumbnails WITH DATA;
  RETURN NEW;
END;
$_$
LANGUAGE plpgsql;

DROP TRIGGER IF EXISTS refreshInstanceThumbnails
ON med_img.Instance;

CREATE TRIGGER refreshInstanceThumbnails
  AFTER INSERT OR UPDATE OR DELETE OR TRUNCATE
  ON med_img.Instance
EXECUTE PROCEDURE med_img.refreshInstanceThumbnails();
-- foreign keys
-- Reference: Hospital_Location (table: Hospital)
ALTER TABLE med_img.Hospital ADD CONSTRAINT Hospital_Location
    FOREIGN KEY (location_id)
    REFERENCES med_img.Location (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Image_Series (table: Instance)
ALTER TABLE med_img.Instance ADD CONSTRAINT Image_Series
    FOREIGN KEY (series_uuid)
    REFERENCES med_img.Series (uuid)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Offers_Hospital (table: Offers)
ALTER TABLE med_img.Offers ADD CONSTRAINT Offers_Hospital
    FOREIGN KEY (hospital_id)
    REFERENCES med_img.Hospital (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Offers_Services (table: Offers)
ALTER TABLE med_img.Offers ADD CONSTRAINT Offers_Services
    FOREIGN KEY (services_id)
    REFERENCES med_img.Services (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Region_Instance (table: Region)
ALTER TABLE med_img.Region ADD CONSTRAINT Region_Instance
    FOREIGN KEY (instance_uuid)
    REFERENCES med_img.Instance (uuid)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Schedule_Hospital (table: Schedule)
ALTER TABLE med_img.Schedule ADD CONSTRAINT Schedule_Hospital
    FOREIGN KEY (hospital_id)
    REFERENCES med_img.Hospital (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Series_Hospital (table: Series)
ALTER TABLE med_img.Series ADD CONSTRAINT Series_Hospital
    FOREIGN KEY (hospital_id)
    REFERENCES med_img.Hospital (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Series_Study (table: Series)
ALTER TABLE med_img.Series ADD CONSTRAINT Series_Study
    FOREIGN KEY (study_uuid)
    REFERENCES med_img.Study (uuid)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Study_Hospital (table: Study)
ALTER TABLE med_img.Study ADD CONSTRAINT Study_Hospital
    FOREIGN KEY (hospital_id)
    REFERENCES med_img.Hospital (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Study_Patient (table: Study)
ALTER TABLE med_img.Study ADD CONSTRAINT Study_Patient
    FOREIGN KEY (patient_uuid)
    REFERENCES med_img.Patient (uuid)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Treats_Hospital (table: Treats)
ALTER TABLE med_img.Treats ADD CONSTRAINT Treats_Hospital
    FOREIGN KEY (hospital_id)
    REFERENCES med_img.Hospital (id)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- Reference: Treats_Patient (table: Treats)
ALTER TABLE med_img.Treats ADD CONSTRAINT Treats_Patient
    FOREIGN KEY (patient_uuid)
    REFERENCES med_img.Patient (uuid)
    NOT DEFERRABLE
    INITIALLY IMMEDIATE
;

-- End of file.

